#
#--------------------------------------------------------------------
#   Include
#--------------------------------------------------------------------
import math as MT
import numpy as NP
#
#--------------------------------------------------------------------
#   Helper
#--------------------------------------------------------------------
def RadDeg(rad):
    return rad * 180.0 / NP.pi
def DegRad(deg):
    return deg * NP.pi / 180.0
#
#--------------------------------------------------------------------
#   Solver
#--------------------------------------------------------------------
def SolverGeneralQuadraticEquation(a, b, c):
    D = b * b - 4 * a * c
    X1 = 0.0
    X2 = 0.0
    if (D < 0):
       return 0, X1, X2
    if (0 == D):
       X1 = -b / 2 / a
       X1 = X2
       return 1, X1, X2
    # 0 < D
    X1 = -(b + MT.sqrt(D)) / 2 / a
    X2 = -(b - MT.sqrt(D)) / 2 / a
    return 2, X1, X2
#
def SolverNormalizedQuadraticEquation(p, q):
    D = 0.25 * p * p - q
    X1 = 0.0
    X2 = 0.0
    if (D < 0):
       return 0, X1, X2
    if (0 == D):
       X1 = -p / 2
       X1 = X2
       return 1, X1, X2
    # 0 < D
    X1 = -p / 2 + MT.sqrt(D)
    X2 = -p / 2 - MT.sqrt(D)
    return 2, X1, X2
#
#--------------------------------------------------------------------
#   Main
#--------------------------------------------------------------------
if ('__main__' == __name__):
    print('*** ConicSection: begin')   
    #
    # 0 = a x^2 + b xy + c y^2 + d x +e y + f
    #-----------------------------------------------------------
    # # 0 = 5 x^2 - 4 xy + 8 y^2 + 20/sqrt(5)x - 80/sqrt(5)y + 4
    # a =  +5
    # b =  -4
    # c =  +8
    # d = +20 / MT.sqrt(5)
    # e = -80 / MT.sqrt(5)
    # f =  +4
    #-----------------------------------------------------------
    a =  +41
    b =  +24
    c =  +34
    d = +110
    e =  +20
    f =  +50
    
    print('a[%+.3f]' % a)
    print('b[%+.3f]' % b)
    print('c[%+.3f]' % c)
    print('d[%+.3f]' % d)
    print('e[%+.3f]' % e)
    print('f[%+.3f]' % f)
    #
    if (a != c):
        Phi = 0.5 * MT.atan2(b, a - c)
    else:
        Phi = 0.5 * NP.sign(b) * NP.pi
    
    print('Phi[%+.3f]rad' % Phi)
    print('Phi[%+.3f]deg' % RadDeg(Phi))

    #----------------------------------------
    CP = MT.cos(Phi)
    SP = MT.sin(Phi)
    A = a * CP * CP + b * CP * SP + c * SP * SP
    C = a * SP * SP - b * CP * SP + c * CP * CP
    S = d * CP + e * SP
    T = e * CP - d * SP
    K = -f
    #----------------------------------------
    Xm = -S / 2 / A
    Ym = -T / 2 / C
    N = C * S * S + A * T * T + 4 * A * C * K
    Al = MT.sqrt(N / 4 / A / A / C)
    As = MT.sqrt(N / 4 / A / C / C)
    #----------------------------------------
    print('Xm[%+.3f]' % Xm)
    print('Ym[%+.3f]' % Ym)
    print('Amajor[%+.3f]' % Al)
    print('Aminor[%+.3f]' % As)
    #----------------------------------------   
    #    
    #----------------------------------------
    #   Test:
    #----------------------------------------
    NC = 36
    for T in range(0, NC):
        WA = MT.pi * T / NC
        x = Al * MT.cos(WA)
        y = As * MT.sin(WA)
        
    #----------------------------------------
    print('*** ConicSection: end')
    #
#